package net.minecraftforge.fluids;

import net.minecraft.block.BlockDispenser;
import net.minecraft.dispenser.BehaviorDefaultDispenseItem;
import net.minecraft.dispenser.IBlockSource;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntityDispenser;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.fluids.capability.IFluidHandler;

/**
 * Fills or drains a fluid container item using a Dispenser.
 */
public class DispenseFluidContainer extends BehaviorDefaultDispenseItem
{
    private static final DispenseFluidContainer INSTANCE = new DispenseFluidContainer();

    public static DispenseFluidContainer getInstance()
    {
        return INSTANCE;
    }

    private DispenseFluidContainer() {}

    private final BehaviorDefaultDispenseItem dispenseBehavior = new BehaviorDefaultDispenseItem();

    public ItemStack func_82487_b(IBlockSource source, ItemStack stack)
    {
        if (FluidUtil.getFluidContained(stack) != null)
        {
            return dumpContainer(source, stack);
        }
        else
        {
            return fillContainer(source, stack);
        }
    }

    /**
     * Picks up fluid in front of a Dispenser and fills a container with it.
     */
    private ItemStack fillContainer(IBlockSource source, ItemStack stack)
    {
        World world = source.func_82618_k();
        EnumFacing dispenserFacing = BlockDispenser.func_149937_b(source.func_82620_h());
        BlockPos blockpos = source.func_180699_d().func_177972_a(dispenserFacing);

        ItemStack result = FluidUtil.tryPickUpFluid(stack, null, world, blockpos, dispenserFacing.func_176734_d());
        if (result == null)
        {
            return super.func_82487_b(source, stack);
        }

        world.func_175698_g(blockpos);

        if (--stack.field_77994_a == 0)
        {
            stack.deserializeNBT(result.serializeNBT());
        }
        else if (((TileEntityDispenser)source.func_150835_j()).func_146019_a(result) < 0)
        {
            this.dispenseBehavior.func_82482_a(source, result);
        }

        return stack;
    }

    /**
     * Drains a filled container and places the fluid in front of the Dispenser.
     */
    private ItemStack dumpContainer(IBlockSource source, ItemStack stack)
    {
        ItemStack dispensedStack = stack.func_77946_l();
        dispensedStack.field_77994_a = 1;
        IFluidHandler fluidHandler = FluidUtil.getFluidHandler(dispensedStack);
        if (fluidHandler == null)
        {
            return super.func_82487_b(source, stack);
        }

        FluidStack fluidStack = fluidHandler.drain(Fluid.BUCKET_VOLUME, false);
        EnumFacing dispenserFacing = BlockDispenser.func_149937_b(source.func_82620_h());
        BlockPos blockpos = source.func_180699_d().func_177972_a(dispenserFacing);

        if (fluidStack != null && fluidStack.amount == Fluid.BUCKET_VOLUME && FluidUtil.tryPlaceFluid(null, source.func_82618_k(), fluidStack, blockpos))
        {
            fluidHandler.drain(Fluid.BUCKET_VOLUME, true);

            if (--stack.field_77994_a == 0)
            {
                stack.deserializeNBT(dispensedStack.serializeNBT());
            }
            else if (((TileEntityDispenser)source.func_150835_j()).func_146019_a(dispensedStack) < 0)
            {
                this.dispenseBehavior.func_82482_a(source, dispensedStack);
            }

            return stack;
        }
        else
        {
            return this.dispenseBehavior.func_82482_a(source, stack);
        }
    }
}
